﻿using MaplePacketLib2.Tools;
using MapleServer2.Constants;
using MapleServer2.Types;

namespace MapleServer2.Packets;

// Whenever server sends 02, client makes a request
//
// 01 and 03 seem to be the first ones sent after entering game
// perhaps setting some initial state?
public static class TimeSyncPacket
{
    private enum Mode : byte
    {
        SetSessionServerTick = 0x0,
        SetInitial1 = 0x1,
        Request = 0x2,
        SetInitial2 = 0x3
    }

    public static PacketWriter SetSessionServerTick(int key)
    {
        PacketWriter pWriter = PacketWriter.Of(SendOp.ResponseTimeSync);
        pWriter.Write(Mode.SetSessionServerTick);
        pWriter.WriteInt(Environment.TickCount);
        pWriter.WriteLong(TimeInfo.Now());
        pWriter.WriteByte();
        pWriter.WriteInt();
        pWriter.WriteInt(key);

        return pWriter;
    }

    public static PacketWriter SetInitial1()
    {
        PacketWriter pWriter = PacketWriter.Of(SendOp.ResponseTimeSync);
        pWriter.Write(Mode.SetInitial1);
        pWriter.WriteInt(Environment.TickCount);
        pWriter.WriteLong(TimeInfo.Now());
        pWriter.WriteByte();
        pWriter.WriteInt();

        return pWriter;
    }

    public static PacketWriter Request()
    {
        PacketWriter pWriter = PacketWriter.Of(SendOp.ResponseTimeSync);
        pWriter.Write(Mode.Request);
        pWriter.WriteInt(Environment.TickCount);
        pWriter.WriteLong(TimeInfo.Now());
        pWriter.WriteByte();
        pWriter.WriteInt();

        return pWriter;
    }

    public static PacketWriter SetInitial2()
    {
        PacketWriter pWriter = PacketWriter.Of(SendOp.ResponseTimeSync);
        pWriter.Write(Mode.SetInitial2);
        pWriter.WriteLong(TimeInfo.Now());

        return pWriter;
    }
}
